import React, {
  useEffect,
  useMemo,
  useRef,
  useState,
  forwardRef,
  useImperativeHandle,
} from 'react';
import { Card } from 'antd';
import SDK from './sdk';

export default forwardRef((props, ref) => {
  const id = useMemo(() => `hatc-${Math.random().toString(36).substring(2, 8)}`, []);

  const { previewParams, width, height } = props;
  const ctrlRef = useRef();
  const [inited, setInited] = useState(false);
  const [loading, setLoading] = useState(false);

  useEffect(() => {
    setLoading(true);
    SDK({ id, width, height }).then((res) => {
      ctrlRef.current = res;
      setInited(true);
      setLoading(false);
    });

    return () => {
      if (ctrlRef.current) {
        ctrlRef.current.destroy();
      }
    };
  }, [id, width, height]);

  useEffect(() => {
    if (inited) {
      if (previewParams) {
        setLoading(true);
        ctrlRef.current
          .preview(previewParams)
          .then(() => {
            setLoading(false);
          })
          .catch((err) => {
            setLoading(false);
            console.log('-------err----->', err);
          });
      }
    }
  }, [inited, previewParams]);

  useImperativeHandle(ref, () => ({
    stop: () => {
      if (ctrlRef.current) {
        ctrlRef.current.stopPreview();
      }
    },
  }));

  return <div id={id} />;
});
